#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/param.h>
#include <ss.h>
#include <flask_util.h>
#include <fs_secure.h>

/* Written by Russell Coker <russell@coker.com.au> */

/* The following header is for struct devfsd_notify_struct */
#include <linux/devfs_fs.h>

security_id_t initial_sid = -1;

int set_sid(char *name, char *sid_name)
{
  security_id_t sid;
  struct stat buf;
  const char * const initial_context = "system_u:object_r:device_t";
  const char * const no_context = "system_u:object_r:unlabeled_t";

  if(is_flask_enabled())
  {
    if(stat_secure(name, &buf, &sid))
    {
      syslog(LOG_ERR, "Can't stat %s: %m", name);
      return 0;
    }
    if(initial_sid == -1)
    {
      if(security_context_to_sid((security_context_t)initial_context
       , strlen(initial_context) + 1
       , &initial_sid))
      {
        syslog(LOG_ERR, "bad sid: \"%s\".\n", initial_context);
        return 0;
      }
    }
    if(sid != initial_sid)
    {
      char sbuf[256];
      unsigned int sbuflen = sizeof(sbuf);
      if(!security_sid_to_context(sid, (security_context_t)sbuf, &sbuflen))
      {
	if(strcmp(sbuf, no_context))
	{
	  syslog(LOG_INFO, "Node %s already has a security context - no change", name);
	  return 0;
	}
      }
    }
    if(security_context_to_sid(sid_name, strlen(sid_name) + 1, &sid))
    {
      syslog(LOG_ERR, "bad sid: \"%s\".\n", sid_name);
      return 0;
    }
    if(chsid(name, sid))
    {
      syslog(LOG_ERR, "error changing SID for %s: %m\n", name);
      return 0;
    }
  }

  return 0;
}
