/* 
 * get_user_sids.c
 *
 * Display the set of SIDs for legal security contexts 
 * for a given user that can be reached by a specified SID.
 */

#include <ss.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <get_sid_list.h>

static void nomem();

#define CONTEXT_LEN 255
#define SIDS_NEL 25

int main(int argc, char **argv)
{
	int ret_val, i, nel;
	char *username;
	unsigned int namelen;
	security_id_t fromsid, user_sid;
	security_id_t *sids;
	security_context_t context;
	unsigned int contextlen;

	if (argc != 3) {
		fprintf(stderr,
			"usage:  %s fromsid user OR\n\t%s fromcontext user\n",
			argv[0], argv[0]);
		exit(1);
	}

	fromsid = atoi(argv[1]);
	if (fromsid == 0) {
		ret_val =
		    security_context_to_sid(argv[1], strlen(argv[1]) + 1,
					    &fromsid);
		if (ret_val) {
			fprintf(stderr, "%s:  invalid SID or context %s\n",
				argv[0], argv[1]);
			exit(1);
		}
	}

	username = argv[2];
	namelen = strlen(argv[2]) + 1;

	nel = SIDS_NEL;
	sids = malloc(sizeof(security_id_t) * nel);
	if (!sids) {
		fprintf(stderr, "%s:  out of memory\n", argv[0]);
		exit(1);
	}

	ret_val = get_ordered_sid_list(username, fromsid, sids, &nel);
	if (ret_val <= 0 && nel > SIDS_NEL) {
		security_id_t *tmplist;
		tmplist = realloc(sids, nel * sizeof(security_id_t));
		if (tmplist) {
			sids = tmplist;
			ret_val = get_ordered_sid_list(username, fromsid, sids, &nel);
		}
	}

	if (ret_val <= 0) {
	    fprintf(stderr, "unable to obtain SIDs for %s.\n", username);
            if (manual_user_enter_sid (username, &user_sid)) {
		    fprintf (stderr, "unable to get valid sid FOR %s", username);
		    exit(1);
	    }
	} else {
            query_user_sid (sids, ret_val, &user_sid);
	}

	free (sids);

	contextlen = CONTEXT_LEN;
	context = malloc(CONTEXT_LEN);
	if (!context) {
		fprintf(stderr, "%s:  out of memory\n", argv[0]);
		exit(1);
	}
	memset(context, 0, contextlen);
	ret_val =
		security_sid_to_context(user_sid, context, &contextlen);
	if (ret_val && (contextlen > CONTEXT_LEN)) {
		security_context_t tmpcon;
		tmpcon = realloc(context, contextlen);
		if (tmpcon) {
			context = tmpcon;
			ret_val =
				security_sid_to_context(user_sid, context,
							&contextlen);
		}
	}
	if (ret_val) {
		perror("security_sid_to_context");
		exit(1);
	}
	printf("\nSelected %s.\n", context);
	free(context);
	exit(0);
}
