#############################################################
#  analysis_tab.tcl  
# -----------------------------------------------------------
#  Copyright (C) 2003 Tresys Technology, LLC
#  see file 'COPYING' for use and warranty information 
#
#  Requires tcl and tk 8.3+, with BWidgets
#  Author: <don.patterson@tresys.com>
# -----------------------------------------------------------

##############################################################
# ::Apol_Analysis
#  
# The Analysis tab
##############################################################
namespace eval Apol_Analysis {
	# Global widgets 
	variable results_notebook
	variable analysis_listbox
	variable opts_frame
	variable updateButton
	variable popupTab_Menu
	
	# Other
	variable analysis_modules	""
	variable curr_analysis_module	""
	variable raised_tab_analysis_type ""
	
	# VARIABLES FOR INSERTING AND DELETING RESULTS TABS
	variable totalTabCount		10
	variable currTabCount		0
	variable pageNums		0
	variable tabName		"ResultsTab"
	variable tabText		"Results "
	variable pageID			""	
	variable results		""
	variable enableUpdate		0
	variable initTab		0
	variable tab_deleted_flag	0
	variable content_array
}

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::mod_select
# ------------------------------------------------------------------------------
proc Apol_Analysis::mod_select { mod_name } {     
	variable opts_frame
	variable curr_analysis_module
	variable analysis_listbox
	variable raised_tab_analysis_type
	variable content_array
	variable results_notebook
	
	# Return if the user selects the same analysis type currently selected	
	if { $mod_name == $curr_analysis_module } {
		return 
	} 	
	# Set the selection and update the current analysis module variable
	$analysis_listbox selection set $mod_name
     	set curr_analysis_module $mod_name
     	
	# Redraw the options frame 
	Apol_Analysis::clear_options_frame $opts_frame
     	Apol_Analysis::display_mod_options $mod_name $opts_frame
     	
     	# If this analysis module matches the stored module for the currently raised tab, update the 
     	# display to reflect the tabs' current state. 
     	if { $raised_tab_analysis_type == $curr_analysis_module } {
     		set raised [$results_notebook raise]
     		set options $content_array($raised,options)
     		Apol_Analysis::set_display_to_results_state $curr_analysis_module $options
     	}
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::delete_ResultsTab
# ------------------------------------------------------------------------------
proc Apol_Analysis::delete_ResultsTab { pageID } {
	variable results_notebook
	variable currTabCount
	variable tab_deleted_flag
	variable content_array
	variable curr_analysis_module
	variable opts_frame

	# If there are tabs available ...
	if { $currTabCount > 0 } {
		# If this is the last tab ...
		if { $currTabCount == 1 } {
			Apol_Analysis::reset_to_initial_state 
			${curr_analysis_module}::clear_results
		} else {
			# Get previous page index.
			set prevPageIdx [expr [$results_notebook index $pageID] - 1]
			# Remove tab and its' widgets; then decrement current tab counter
			$results_notebook delete $pageID 
    			set currTabCount [expr $currTabCount - 1]
     			
    			# Raise the previous page. If the first tab was deleted then raise the current first tab.
			if { $prevPageIdx >= 0 } { 
				set raised [$results_notebook raise [$results_notebook page $prevPageIdx]]
			} else { 
				set raised [$results_notebook raise [$results_notebook page 0]]
			}
    			# Set the search option widgets according to the now raised results tab.
			# Only redraw the options frame if this is a new analysis type. 
			if { $content_array($raised,mod_name) != $content_array($pageID,mod_name) } {
				set curr_analysis_module $content_array($raised,mod_name)
		     		Apol_Analysis::clear_options_frame $opts_frame
		     		Apol_Analysis::display_mod_options $curr_analysis_module $opts_frame 	 	
		     	}
		     	# Remove the deleted tabs information from the content array
    			array unset content_array($pageID)
    			set tab_deleted_flag 1
    			Apol_Analysis::switch_results_tab $raised
    			set tab_deleted_flag 0
    			# Update options to the current tab.
		     	#set options $content_array($raised,options)
		     	#Apol_Analysis::set_display_to_results_state $curr_analysis_module $options 
    		}
	} 
     		
    	return 0
}

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::close_All_ResultsTabs
# ------------------------------------------------------------------------------
proc Apol_Analysis::close_All_ResultsTabs { } {
	variable content_array
	variable results_notebook
	variable currTabCount
		
	# 1. Unset the entire content_array, which is used to gather the options for each search result.
        # 2. Retrieve all existing tabs and set it to our local tabList variable.
	array unset content_array
        set tabList [$results_notebook pages 0 [expr $currTabCount - 1]]
        
        # The following 'for loop' goes through each tab in the local tabList variable
        # and deletes each tab.
        foreach tab $tabList {
	    	$results_notebook delete $tab
	} 
					
	# Reset the result tab variables.
	set Apol_Analysis::pageNums 		0
	set Apol_Analysis::currTabCount		0
	set Apol_Analysis::pageID		""	
	set Apol_Analysis::results		""
	set Apol_Analysis::initTab		0
	set Apol_Analysis::enableUpdate 	0		
        return 0
}

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::clear_results_frame
# ------------------------------------------------------------------------------
proc Apol_Analysis::clear_results_frame { results_frame } {  
	set parent [winfo parent $results_frame]
	destroy $results_frame
	Apol_Analysis::create_results_frame $parent
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::create_results_frame
# ------------------------------------------------------------------------------
proc Apol_Analysis::create_results_frame { parent } {  	
	set tmp [frame $parent.results_frame]
	pack $tmp -side left -fill both -anchor nw -expand yes 
     	return $tmp
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::get_results_frame
# ------------------------------------------------------------------------------
proc Apol_Analysis::get_results_frame { tabID } {  
	variable results_notebook
	set parent [$results_notebook getframe $tabID]	
     	return "$parent.results_frame"
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::create_New_ResultsTab
# ------------------------------------------------------------------------------
proc Apol_Analysis::create_New_ResultsTab { } {
	variable results_notebook
	variable currTabCount
	variable totalTabCount
	variable pageNums
	variable tabName
	variable tabText
	
	if { $currTabCount >= $totalTabCount } {		
		tk_messageBox -icon error -type ok -title "Attention" \
			-message "You have reached the maximum amount of tabs. Please delete a tab and try again."
		return ""
	}
	
	# Increment the global tab count and pageNum variables
	incr currTabCount
    	incr pageNums

	# Create tab and its' widgets
	$results_notebook insert end $tabName$pageNums -text $tabText$pageNums
    	# Create explicit inner frame
    	set tab_frame [$results_notebook getframe $tabName$pageNums]
    	set results_frame [Apol_Analysis::create_results_frame $tab_frame]
    	# Here we use a flag variable to determine if this is the first tab created.
    	# If it is, then we can set the NoteBook size here, instead of repeatedly
    	# computing the notebook size every time ApolTE::create_New_ResultsTab is called.
    	if { $Apol_Analysis::initTab == 0 } {	
    		$results_notebook compute_size
    		set Apol_Analysis::initTab 1
    	}   
    					
    	# Raise the new tab
    	set newPageIdx 	[expr $currTabCount - 1]
    	set raisedPage 	[$results_notebook raise [$results_notebook page $newPageIdx]]
	     	
    	return $results_frame
}

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::create_options_frame
# ------------------------------------------------------------------------------
proc Apol_Analysis::create_options_frame { parent } {  	
	set tmp [frame $parent.inner_opt_frame]
	pack $tmp -side left -fill both -anchor nw -expand yes 
     	return $tmp
} 


# ------------------------------------------------------------------------------
#  Command Apol_Analysis::clear_options_frame
# ------------------------------------------------------------------------------
proc Apol_Analysis::clear_options_frame { opts_frame } {  
	set parent [winfo parent $opts_frame]
	destroy $opts_frame
	Apol_Analysis::create_options_frame $parent
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::switch_results_tab
# ------------------------------------------------------------------------------
proc Apol_Analysis::switch_results_tab { tabID } {   
	variable opts_frame
	variable content_array
	variable results_notebook
	variable tab_deleted_flag
	variable curr_analysis_module
     	variable raised_tab_analysis_type 
     	
     	set raised [$results_notebook raise]

	# First check flag to determine if the user has simply selected the 
	# currently raised tab and if so just return without updating options.
	if { $raised == $tabID && $tab_deleted_flag == 0 } {
		return
	} 
	# Only redraw the options frame if this is a new analysis type. 
	if { $content_array($raised,mod_name) != $content_array($tabID,mod_name) } {
		set curr_analysis_module $content_array($tabID,mod_name)
     		Apol_Analysis::clear_options_frame $opts_frame
     		Apol_Analysis::display_mod_options $curr_analysis_module $opts_frame 	 	
     	}
     	Apol_Analysis::set_display_to_results_state $curr_analysis_module $content_array($tabID,options) 
     	set raised_tab_analysis_type $curr_analysis_module
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::store_current_results_state
# ------------------------------------------------------------------------------
proc Apol_Analysis::store_current_results_state { raisedPage } {     	
	variable curr_analysis_module
     	variable content_array

     	set options [Apol_Analysis::get_current_results_state]
	# Unsets all of the elements in the array that match $raisedPage 
	array unset content_array $raisedPage
     	set content_array($raisedPage,options) $options
     	set content_array($raisedPage,mod_name) $curr_analysis_module
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::display_new_content
# ------------------------------------------------------------------------------
proc Apol_Analysis::display_new_content { } {     	
     	
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::display_mod_options
# ------------------------------------------------------------------------------
proc Apol_Analysis::display_mod_options { mod_name opts_frame } { 
     	${mod_name}::display_mod_options $opts_frame
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::reset_results_options
# ------------------------------------------------------------------------------
proc Apol_Analysis::reset_results_options { } {     	
     	
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::close_results_tab
# ------------------------------------------------------------------------------
proc Apol_Analysis::close_results_tab { } {     	
     	
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::remove_from_content_array
# ------------------------------------------------------------------------------
proc Apol_Analysis::remove_from_content_array { } {     	
     	
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::remove_tab
# ------------------------------------------------------------------------------
proc Apol_Analysis::remove_tab { } {     	
     	
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::get_current_results_state
# ------------------------------------------------------------------------------
proc Apol_Analysis::get_current_results_state { } {     	
     	variable curr_analysis_module
     	
     	return [${curr_analysis_module}::get_current_results_state]
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::set_display_to_results_state
# ------------------------------------------------------------------------------
proc Apol_Analysis::set_display_to_results_state { mod_name options } { 
	variable analysis_listbox
	$analysis_listbox selection set $mod_name    	
     	${mod_name}::set_display_to_results_state $options
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::register_analysis_modules
# ------------------------------------------------------------------------------
proc Apol_Analysis::register_analysis_modules { mod_name desc_name } {     	
     	variable analysis_modules
   	
   	set item_list [list $mod_name "$desc_name" ]
     	set analysis_modules [lappend analysis_modules $item_list]
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::do_analysis
# ------------------------------------------------------------------------------
proc Apol_Analysis::do_analysis { which } {
	variable results_notebook
    	variable totalTabCount
    	variable currTabCount
	variable enableUpdate
	variable curr_analysis_module
	variable raised_tab_analysis_type
	variable analysis_listbox

	if { $curr_analysis_module == "" } {
		tk_messageBox -icon error -type ok -title "Error" \
			-message "You must select an analysis type."
		return 
	}
							
	ApolTop::setBusyCursor
	switch $which {
		new_analysis {
			# If the update button is disabled, then enable it.
			if { $enableUpdate == 0 } {
				$Apol_Analysis::updateButton configure -state normal
				set enableUpdate 1
			}
			# Create the new results tab.
			set results_frame [Apol_Analysis::create_New_ResultsTab]
		}
		update_analysis {
			set results_frame [Apol_Analysis::get_results_frame [ $results_notebook raise ]]
			Apol_Analysis::clear_results_frame $results_frame
		}
		default {
			return -code error
		}
	} 
	if { $results_frame != "" } {
		set rt [catch {${curr_analysis_module}::do_analysis $results_frame} err] 
		# Here store the current content since we are not switching tabs, even if there is an
		# error performing the analysis. The user should still be able to enter the correct 
		# arguments and then select update.
		Apol_Analysis::store_current_results_state [$results_notebook raise]
		
		if { $rt != 0 } {	
			ApolTop::resetBusyCursor
			set err_label [label $results_frame.err_label \
				-text "[$analysis_listbox itemcget $curr_analysis_module -text] analysis error:\n\n$err" \
				-justify center -font { Helvetica 14 bold } -foreground red]
			pack $err_label -side left -anchor n -fill both -expand yes
			return -1
		} 
		set raised_tab_analysis_type $curr_analysis_module
	}
        ApolTop::resetBusyCursor
        
     	return 
} 

###############################################################################
# ::order_analysis_listbox
#
proc Apol_Analysis::order_analysis_listbox { analysis_listbox } {		
	#get modules from list	
	set labels ""		
	foreach module [$analysis_listbox items] {
		lappend labels "{[$analysis_listbox itemcget $module -text]} {$module}"
	}
	set labels [lsort -dictionary $labels]
	
	set module_List ""
	foreach module $labels {
		lappend module_List [lindex $module end]
	}
	$analysis_listbox reorder $module_List

	return 0
}

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::configure_analysis_listbox
# ------------------------------------------------------------------------------
proc Apol_Analysis::configure_analysis_listbox { analysis_modules analysis_listbox } {      	
     	foreach mod_name $analysis_modules { 
		$analysis_listbox insert end [lindex $mod_name 0] \
			-text [lindex $mod_name 1] 
	}    
	#Apol_Analysis::order_analysis_listbox $analysis_listbox
	# Redraw the listbox
	$analysis_listbox configure -redraw 1
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::initialize
# ------------------------------------------------------------------------------
proc Apol_Analysis::initialize { } {     	
     	variable analysis_modules
     	variable analysis_listbox
    	
    	# Register any analysis modules
     	foreach mod_name $analysis_modules {
     		set mod_name [lindex $mod_name 0]
     		${mod_name}::initialize
     	}
     	Apol_Analysis::configure_analysis_listbox $analysis_modules $analysis_listbox
     	$analysis_listbox selection set [$analysis_listbox items 0]
	if { [$analysis_listbox selection get] != "" } {
		Apol_Analysis::mod_select [$analysis_listbox selection get]	
	}
	
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::reset_to_initial_state
# ------------------------------------------------------------------------------
proc Apol_Analysis::reset_to_initial_state { } {	 	
    	$Apol_Analysis::updateButton configure -state disabled
    	Apol_Analysis::close_All_ResultsTabs
     	set Apol_Analysis::raised_tab_analysis_type ""
     	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::open
# ------------------------------------------------------------------------------
proc Apol_Analysis::open { } { 
	variable analysis_listbox
	if { [$analysis_listbox selection get] != "" } {
		[$analysis_listbox selection get]::open	
	}
	return 0
} 

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::close
# ------------------------------------------------------------------------------
proc Apol_Analysis::close { } {	
	variable analysis_modules
    	
     	foreach mod_name $analysis_modules {
     		set mod_name [lindex $mod_name 0]
     		${mod_name}::close
     	}
	Apol_Analysis::reset_to_initial_state	  
    	return 0	
}

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::discard_analysis_modules
# ------------------------------------------------------------------------------
proc Apol_Analysis::discard_analysis_modules { } {	
	variable analysis_modules
    	set analysis_modules ""
    	return 0	
}

########################################################################
#  Command Apol_Analysis::goto_line
#  	- goes to indicated line in text box
# 
proc Apol_Analysis::goto_line { line_num } {
	return 0
}

##############################################################
#  Command Apol_Analysis::search
#  	- Search text widget for a string
# 
proc Apol_Analysis::search { str case_Insensitive regExpr srch_Direction } {
	return 0
}

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::popupResultsTab_Menu
# ------------------------------------------------------------------------------
proc Apol_Analysis::popupResultsTab_Menu { window x y popupMenu page } {
	variable pageID
	set pageID $page
        # Getting global coordinates of the application window (of position 0, 0)
	set gx [winfo rootx $window]	
	set gy [winfo rooty $window]
	
	# Add the global coordinates for the application window to the current mouse coordinates
	# of %x & %y
	set cmx [expr $gx + $x]
	set cmy [expr $gy + $y]

	# Posting the popup menu
   	tk_popup $popupMenu $cmx $cmy
   	
   	return 0
}

# ------------------------------------------------------------------------------
#  Command Apol_Analysis::create
# ------------------------------------------------------------------------------
proc Apol_Analysis::create { nb } {
	variable results_notebook
	variable analysis_listbox
	variable opts_frame
	variable updateButton
	variable popupTab_Menu
	
	# Layout frames
	set frame [$nb insert end $ApolTop::analysis_tab -text "Analysis"]
	set topf  [frame $frame.topf]
	set botf  [frame $frame.botf]
		
	# Major subframes
	set t_left_f   [TitleFrame $topf.t_left_f -text "Analysis Type"]
	set title_opts_f [TitleFrame $topf.opts_frame -text "Analysis Options"]
	set buttons_f [frame $topf.buttons_f]
	set b_title_f [TitleFrame $botf.b_title_f -text "Analysis Results"]
	set b_topf [frame [$b_title_f getframe].b_topf]
	set b_botf [frame [$b_title_f getframe].b_botf -relief sunken -bd 1]
	
	# Placing layout frames and major subframes
	pack $t_left_f -side left -anchor nw -fill y
	pack $title_opts_f -side left -fill both -anchor nw -expand yes -padx 2
	pack $buttons_f -side left -fill y -anchor nw -padx 2 -pady 2
	pack $b_title_f -side left -fill both -anchor n -expand yes
	pack $b_topf -side top -fill both -anchor nw -expand yes
	pack $b_botf -side bottom -anchor center -fill x -padx 4 -pady 1	
	pack $topf -side top -fill x 
	pack $botf -side top -fill both -expand yes 
	
	set opts_frame [Apol_Analysis::create_options_frame [$title_opts_f getframe]]
	
	# Analysis type listbox
	set sw_t     [ScrolledWindow [$t_left_f getframe].sw -auto none]
	set analysis_listbox [ListBox $sw_t.analysis_listbox \
                  -relief flat -borderwidth 0 -bg white \
                  -height 6 -highlightthickness 2 -width 25 -padx 0 \
                  -redraw 0 -selectmode single]
	$sw_t setwidget $analysis_listbox 
    	
    	$analysis_listbox bindText <ButtonPress-1> { Apol_Analysis::mod_select }
    	
    	# Popup menu widget
	set popupTab_Menu [menu .analysis_popup_Menu]
	$popupTab_Menu add command -label "Delete Tab" \
		-command { Apol_Analysis::delete_ResultsTab $Apol_Analysis::pageID }
	
	# Results notebook
	set results_notebook [NoteBook $b_topf.nb_results]
	$results_notebook bindtabs <Button-3> {Apol_Analysis::popupResultsTab_Menu %W %x %y $Apol_Analysis::popupTab_Menu} 
    	$results_notebook bindtabs <Button-1> {Apol_Analysis::switch_results_tab}
    
	# Action buttons
	set newButton 	 [button $buttons_f.new -text "New" \
		-width 6 \
		-command { Apol_Analysis::do_analysis "new_analysis" }]
	set updateButton [button $buttons_f.upDate -text "Update" \
		-width 6 \
		-command { Apol_Analysis::do_analysis "update_analysis" } \
		-state disabled] 		
    	
    	# Add button bar at bottom of results section for closing tabs.
	set bClose [button $b_botf.bClose -text "Close Tab" -command { 
		Apol_Analysis::delete_ResultsTab [$Apol_Analysis::results_notebook raise] }]
	pack $bClose -side bottom -anchor center -fill x -padx 1 -pady 1
    		   
	# Placing widgets    
	$results_notebook compute_size
	pack $results_notebook -fill both -expand yes -padx 4
    	pack $sw_t -fill both -expand yes
    	pack $newButton $updateButton -side top -pady 5 -anchor ne 
	
	Apol_Analysis::initialize
		    	    
	return $frame	
}

