 /* Copyright (C) 2001-2003 Tresys Technology, LLC
 * see file 'COPYING' for use and warranty information */

/* 
 * Author: mayerf@tresys.com 
 */

/* This file contains the tcl/tk initialization code for apol
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <tcl.h>
#include <tk.h>
#include "../libapol/apol_tcl.h"
#define STARTUP_SCRIPT "apol.tcl"

/* internal global */
char* policy_conf_file;

int Tcl_AppInit(Tcl_Interp *interp)
{
	char *script;
	char *args[2];
	if (Tcl_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	if (Tk_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	/* apolicy packagae initialization */
	if (Apol_Init(interp) == TCL_ERROR) {
     		return TCL_ERROR;
     	}
     	
     	/* find and start the TCL scripts */
     	args[0] = NULL;
     	args[1] = STARTUP_SCRIPT;
     	if(Apol_GetScriptDir(NULL, interp, 2, args) != TCL_OK) {
     		fprintf(stderr, "Error finding TCL script: %s\n", interp->result);
     		Tcl_DeleteInterp(interp);
     		Tcl_Exit(1);
     	}
     	script = (char *)malloc(strlen(interp->result) + strlen(STARTUP_SCRIPT) + 3);
     	if(script == NULL) {
     		fprintf(stderr, "out of memory\n");
     		Tcl_DeleteInterp(interp);
     		Tcl_Exit(1);
     	}
     	sprintf(script, "%s/%s", interp->result, STARTUP_SCRIPT);
	if(Tcl_EvalFile(interp, script) != TCL_OK) {
		fprintf(stderr, "Error in StartScript (%s): %s on line %d\n\nIf %s is set, make sure directory is correct\n", script, interp->result, interp->errorLine, APOL_ENVIRON_VAR_NAME);
	  	Tcl_DeleteInterp(interp);
	  	Tcl_Exit(1);
	}

/* TODO: There is still a lingering problem with this code (to open policy.conf from command line.
 * for now exclude from the build */
#if 0
	/* If a policy.conf file was provided on command line, open it */
	if(policy_conf_file != NULL) {
		char tmp[530];
		Tcl_DString  dstring, *dstr = &dstring;
		if(strlen(policy_conf_file) > 500) {
			fprintf(stderr, "policy.conf file from command line to large---ignored\n");
		}
		else {
			sprintf(tmp, "ApolTop::openPolicyFile %s", policy_conf_file);
			Tcl_UtfToExternalDString(NULL, tmp, strlen(tmp), dstr);
     			/* We'll just ignore errors and let GUI deal with invalid file types */
     			Tcl_Eval(interp, dstr);
     			Tcl_DStringFree(dstr);
     		}
     	}
#endif
     	
	free(script);
    	return TCL_OK;
}

void usage() 
{
/*	fprintf(stdout, "\napol [filename | -h]\n\n");*/
	fprintf(stdout, "\napol [-h]\n\n");
	/*fprintf(stdout, "     filename     name of a policy.conf file to open\n");*/
	fprintf(stdout, "     -h           display this usage message\n\n");
	return;
}

int main(int argc, char *argv[])
{
	Tcl_Interp *interp;
	if(argc > 2) {
		usage();
		exit(1);
	}
	if(argc == 2 && argv[1][0] == '-') {
		usage();
		exit(0);
	} 
	if(argc == 2) {
		/*policy_conf_file = argv[1];
	}
	else {
		policy_conf_file = NULL;	*/
		usage();
		exit(1);
	}	
	interp = Tcl_CreateInterp();
	Tk_MainEx(1, argv, Tcl_AppInit, interp);
	
  	exit(0);
}




