# Copyright (C) 2001-2003 Tresys Technology, LLC
# see file 'COPYING' for use and warranty information 

# TCL/TK GUI for SE Linux policy analysis
# Requires tcl and tk 8.3+, with BWidgets


##############################################################
# ::Apol_Roles
#  
# The Roles page
##############################################################
namespace eval Apol_Roles {
# opts(opt), where opt =
# roles          use roles

	variable opts
	set opts(roles)			1
	set opts(useType)		0
	set opts(showSelection)         all
	variable srchstr ""
	variable role_1ist ""
	variable types_list ""
	
	# Global Widgets
	variable resultsbox
	variable rlistbox
	variable list

}


proc Apol_Roles::open { } {
	variable role_list
    
        set role_list [apol_GetNames roles]
	set role_list [lsort $role_list]
        $Apol_Roles::list configure -values $ApolTypes::typelist
        return
}

proc Apol_Roles::close { } {
	set Apol_Roles::role_list ""
        $Apol_Roles::list configure -values ""
        $Apol_Roles::resultsbox configure -state normal
        $Apol_Roles::resultsbox delete 0.0 end
        ApolTop::makeTextBoxReadOnly $Apol_Roles::resultsbox 
        set Apol_Roles::types_list ""
	return	
}

proc Apol_Roles::popupRoleInfo {which role} {
	set rt [catch {set info [apol_GetSingleRoleInfo $role 1]} err]
	if {$rt != 0} {
		tk_messageBox -icon error -type ok -title "Error" \
			-message "$err"
		return 
	}

	set w .role_infobox
	catch {destroy $w}
	toplevel $w 
	wm title $w "$role"
	set sf [ScrolledWindow $w.sf  -scrollbar both -auto both]
	set f [text [$sf getframe].f -font {helvetica 10} -wrap none -width 35 -height 10]
	$sf setwidget $f
     	button $w.close -text Close -command "catch {destroy $w}" -width 10
     	$f insert 0.0 $info
 	wm geometry $w +50+50
	pack $w.close -side bottom -anchor s -padx 5 -pady 5 
	pack $sf -fill both -expand yes
	return
}

proc Apol_Roles::popupRoleInfoMenu { global x y popup} {
        # Getting global coordinates of the application window (of position 0, 0)
	set gx [winfo rootx $global]	
	set gy [winfo rooty $global]
	
	# Add the global coordinates for the application window to the current mouse coordinates
	# of %x & %y
	set cmx [expr $gx + $x]
	set cmy [expr $gy + $y]

	# Posting the popup menu
   	tk_popup $popup $cmx $cmy
}

##############################################################
# ::search
#  	- Search text widget for a string
# 
proc Apol_Roles::search { str case_Insensitive regExpr srch_Direction } {
	variable resultsbox
	
	ApolTop::textSearch $resultsbox $str $case_Insensitive $regExpr $srch_Direction
	return 0
}

proc Apol_Roles::searchRoles {} {
	variable opts
	variable resultsbox

	update idletasks

	if {$opts(showSelection) == "names"} {
		set name_only 1
	} else {
		set name_only 0
	}
        set rt [catch {set results [apol_GetRolesByType $name_only $opts(useType) \
		$Apol_Roles::types_list]} err]
	if {$rt != 0} {	
		tk_messageBox -icon error -type ok -title "Error" -message "$err"
		return 
	} else {
	    $resultsbox configure -state normal
	    $resultsbox delete 0.0 end
	    $resultsbox insert end $results
	    ApolTop::makeTextBoxReadOnly $resultsbox 
        }
	
	return
}

proc Apol_Roles::enable_type_list { entry } {
    
    if { $Apol_Roles::opts(useType) } {
	$entry configure -state normal -entrybg  white
    } else {
	$entry configure -state disabled -entrybg  $ApolTop::default_bg_color
    }
	
    return 0
}

########################################################################
# ::goto_line
#  	- goes to indicated line in text box
# 
proc Apol_Roles::goto_line { line_num } {
	variable resultsbox
	
	ApolTop::goto_line $line_num $resultsbox
	return 0
}

proc Apol_Roles::create {nb} {
    variable rlistbox 
    variable resultsbox 
    variable srchstr 
    variable opts
    variable types_list
    variable list
    global tcl_platform
    
    # Layout frames
    set frame [$nb insert end $ApolTop::roles_tab -text "Roles"]
    set topf  [frame $frame.topf]
    set pw1   [PanedWindow $topf.pw -side top]
    set pane  [$pw1 add ]
    set spane [$pw1 add -weight 5]
    set pw2   [PanedWindow $pane.pw -side left]
    set rpane [$pw2 add -weight 3]

    # Title frames
    set rolebox [TitleFrame $rpane.rolebox -text "Roles"]
    set s_optionsbox [TitleFrame $spane.obox -text "Search Options"]
    set resultsbox [TitleFrame $spane.rbox -text "Search Results"]

    # Placing layout
    pack $topf -fill both -expand yes 
    pack $pw1 -fill both -expand yes
    pack $pw2 -fill both -expand yes

    # Placing title frames
    pack $s_optionsbox -padx 2 -fill both
    pack $rolebox -padx 2 -side left -fill both -expand yes
    pack $resultsbox -pady 2 -padx 2 -fill both -anchor n -side bottom -expand yes
   
    # Roles listbox widget
    set sw_r [ScrolledWindow [$rolebox getframe].sw -auto both]
    set rlistbox [listbox [$sw_r getframe].lb -height 18 -width 20 -highlightthickness 0 \
		 -listvar Apol_Roles::role_list] 
    $sw_r setwidget $rlistbox 
    
    # Popup menu widget
    menu .popupMenu_roles
    .popupMenu_roles add command -label "Display Role Info" \
	-command { Apol_Roles::popupRoleInfo "role" [$Apol_Roles::rlistbox get active] }
	    
    # Event bindong on the roles list box widget
    bind $rlistbox <Double-Button-1> { Apol_Roles::popupRoleInfo "role" [$Apol_Roles::rlistbox get active]}
    bind $rlistbox <Button-3> { Apol_Roles::popupRoleInfoMenu %W %x %y .popupMenu_roles }
    
    # Search options subframes
    set ofm [$s_optionsbox getframe]
    set l_innerFrame [LabelFrame $ofm.to \
                    -relief sunken -borderwidth 1]
    set c_innerFrame [LabelFrame $ofm.co \
                    -relief sunken -borderwidth 1]
    set r_innerFrame [frame $ofm.ro \
                    -relief flat -borderwidth 1]
    
    # Placing inner frames
    set lfm [$l_innerFrame getframe]
    set cfm [$c_innerFrame getframe]
    set rfm  $r_innerFrame
    
    # Search options widget items
    set list [ComboBox $cfm.combo -width 30 -textvariable Apol_Roles::types_list \
		  -helptext "Type or select a type"]
    $list configure -state disabled 
    
    # ComboBox is not a simple widget, it is a mega-widget, and bindings for mega-widgets are non-trivial.
    # If bindtags is invoked with only one argument, then the current set of binding tags for window is 
    # returned as a list.
    bindtags $list.e [linsert [bindtags $list.e] 3 listTag]
    bind listTag <KeyPress> { Apol_Users::_create_popup $Apol_Roles::list %W %K }
    
    radiobutton $lfm.names_only -text "Names Only" -variable Apol_Roles::opts(showSelection) -value names 
    radiobutton $lfm.all_info -text "All Information" -variable Apol_Roles::opts(showSelection) -value all 
    checkbutton $cfm.cb -variable Apol_Roles::opts(useType) -text "Search Using Type" \
	-command "Apol_Roles::enable_type_list $list"

    # Action Buttons
    button $rfm.ok -text OK -width 6 -command {Apol_Roles::searchRoles}      
    #button $rfm.print -text Print -width 6 -command {ApolTop::unimplemented}

    # Display results window
    set sw_d [ScrolledWindow [$resultsbox getframe].sw -auto none]
    set resultsbox [text [$sw_d getframe].text -bg white -wrap none -state disabled -font $ApolTop::text_font]
    $sw_d setwidget $resultsbox

    # Placing all widget items
    pack $r_innerFrame -side right -fill both -expand yes -anchor ne
    pack $l_innerFrame -side left -fill both -anchor n
    pack $c_innerFrame -side right -expand yes -anchor nw -padx 5
    pack $rfm.ok -side top -anchor e -pady 5 -padx 5
    pack $lfm.names_only $lfm.all_info -side top -anchor nw -pady 5 -padx 5
    pack $cfm.cb -side top -anchor nw -padx 10 -pady 5
    pack $list -anchor w -pady 10 -padx 10
    pack $sw_r -fill both -expand yes
    pack $sw_d -side left -expand yes -fill both 
        
    return $frame	
}

