/* Copyright (C) 2003 Tresys Technology, LLC
 * see file 'COPYING' for use and warranty information */

/* 
 * Author: mayerf@tresys.com 
 */

/* analysis.h
 *
 * Analysis routines for libapol
 */


#ifndef _APOLICY_ANALYSIS_H_
#define _APOLICY_ANALYSIS_H_

/* These structures are used to return domain transition information.
 * This struucture only captures one level of domain transition...
 * repeated calls are required to build a tree.
 */
 
/* file (program) types that allow appropriate execute and entrypoint perms */
typedef struct entrypoint_type {
	int	src_type;		/* source domain */
	int	tgt_type;		/* target (transitioned) domain */
	int	file_type;		/* index of file type */
	int	num_ep_rules;		/* # of file entrypoint rules for tgt type */
	int	num_ex_rules;		/* # of execute rules for src type */
	int	*ep_rules;		/* array */
	int	*ex_rules;		/* array */
} entrypoint_type_t;
	

/* capture all info for a target domain in a domain trans analysis */
typedef struct tgt_trans_domain {
	int	src_type;		/* index of source type */
	int	tgt_type;		/* index of target type */
	int	num_pt_rules;		/* # of process transition rules */
	int	*pt_rules;		/* dynamic array of pt rules */
	llist_t *entry_types;		/* list of entrypoint types */
} tgt_trans_domain_t;

/* top level domain analysis list */ 
typedef struct domain_trans_analysis {
	int	src_type;		/* index of source type (starting domain) */
	llist_t *tgt_domains; 		/* list of target domains */
} domain_trans_analysis_t;



void free_entrypoint_type(void *t);
void free_tgt_trans_domain(void *t);
void free_domain_trans_analysis(domain_trans_analysis_t *p);
domain_trans_analysis_t *new_domain_trans_analysis(void);
tgt_trans_domain_t *new_tgt_trans_domain(void); 
int determine_domain_trans(char *source, domain_trans_analysis_t **dta, policy_t *policy);


#endif /*_APOLICY_ANALYSIS_H_*/

