/* Copyright (C) 2001, 2002 Tresys Technology, LLC
 * see file 'COPYING' for use and warranty information */

/* 
 * Author: mayerf@tresys.com 
 */

/* avl-util.c
 *
 * Generic support functions for AVL trees.
 */
 
#ifdef USE_SORTING

#include "avl-util.h"
#include "policy.h"
#include<assert.h>

/* single rotated right */
int avl_srl(int head, avl_ptrs_t *ptrs)
{
	int newhead;
	newhead = ptrs[head].left;
	ptrs[head].left = ptrs[newhead].right;
	ptrs[newhead].right = head;
	ptrs[head].height = avl_max_child(head, ptrs) + 1;
	ptrs[newhead].height = avl_max_child(newhead, ptrs) + 1;
	return newhead;
}
/* single rotate right */
int avl_srr(int head, avl_ptrs_t *ptrs)
{
	int newhead;
	newhead = ptrs[head].right;
	ptrs[head].right = ptrs[newhead].left;
	ptrs[newhead].left = head;
	ptrs[head].height = avl_max_child(head, ptrs) + 1;
	ptrs[newhead].height = avl_max_child(newhead, ptrs) + 1;
	return newhead;
}
/* double rotate left */
int avl_drl(int head, avl_ptrs_t *ptrs)
{
	ptrs[head].left = avl_srr(ptrs[head].left, ptrs);
	return avl_srl(head, ptrs);
}
/* double rotate right */
int avl_drr(int head, avl_ptrs_t *ptrs)
{
	ptrs[head].right = avl_srl(ptrs[head].right, ptrs);
	return avl_srr(head, ptrs);
}


int grow_avl_array(avl_ptrs_t **avl, int sz)
{
	if(avl == NULL || *avl == NULL )
		return -1;
	
	
	*avl = (avl_ptrs_t *)realloc(*avl, (LIST_SZ + sz) * sizeof(avl_ptrs_t));
	if(*avl == NULL) {
		fprintf(stderr, "out of memory\n");
		return -1;
	}
	return 0;
}

/* Searches an avl tree looking for a match to key. This function expects 
 * the caller to provide to helper functions: comp and get_subkey.  The
 * prototypes required for these two functions are:
 *	comp(const char *key, int idx, policy_t *policy)
 *	get_subtree(int idx, int dir, policy_t *policy)
 *
 * comp() compares the key with the idx's name from appropriate list and returns
 * 0 >0 or <0 much like strcmp.
 *
 * get_subtree() returns a idex for idx's LEFT or RIGHT child depending on the 
 * value of dir (which of course can be LEFT or RIGHT!).
 */
int avl_get_idx(const char *key, int head, int (*comp)(const char *, int, policy_t *), 
		int (*get_subtree)(int, int, policy_t *), policy_t *policy)
{
	int cmpval, subtree;

	assert(key != NULL);
	assert(policy != NULL);
	
	if(head < 0)
		return -1; /* no match */
	
	cmpval = (*comp)(key, head, policy);
	if(cmpval == 0) 
		return head; /* found! */
	else if(cmpval < 0) {
		subtree = (*get_subtree)(head, LEFT, policy);
		return avl_get_idx(key, subtree, comp, get_subtree, policy);
	}
	else { /* (cmpval > 0) */
		subtree = (*get_subtree)(head, RIGHT, policy);
		return avl_get_idx(key, subtree, comp, get_subtree, policy);
	}
}

#endif /* USE_SORTING */



