/* Copyright (C) 2002-2003 Tresys Technology, LLC
 * see file 'COPYING' for use and warranty information */

/* 
 * Author: mayerf@tresys.com 
 */

/* Test program for libapol
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <tcl.h>
#include <tk.h>
#include <assert.h>
/* apol lib */
#include "..//policy.h"
#include "..//apol_tcl.h"
#include "..//util.h"
#include "../analysis.h"
#include "../render.h"

/* global TCL interp object */
Tcl_Interp * interp;
extern policy_t *policy; /*apolicy_parse.y, our policy DB */
extern FILE *yyin;
FILE *outfile;

int test_print_ep(entrypoint_type_t *ep, policy_t *policy)
{
	int i;
	char *rule;
	char *file_type;
	extern FILE *outfile;
	
	if(get_type_name(ep->file_type, &file_type, policy) != 0) {
		fprintf(stderr, "\nproblem translating file_type (%d)\n", ep->file_type);
		return -1;
	}
	fprintf(outfile, "\n\t     %s (%d):\n", file_type, ep->file_type);
	free(file_type);
	
	fprintf(outfile, "\t          FILE ENTRYPOINT ACCESS RULES (%d rules):\n", ep->num_ep_rules);
	for(i = 0; i < ep->num_ep_rules; i++) {
		rule = re_render_av_rule(0,ep->ep_rules[i], 0, policy);
		if(rule == NULL) {
			fprintf(stderr, "problem rendering entrypoint rule %d\n", i);
			return -1;
		}
		fprintf(outfile, "\t          (%d) %s\n", get_rule_lineno(ep->ep_rules[i],RULE_TE_ALLOW, policy),rule);
		free(rule);
	}
	fprintf(outfile, "\n\t          FILE EXECUTE ACCESS RULES (%d rules):\n", ep->num_ex_rules);
	for(i = 0; i < ep->num_ex_rules; i++) {
		rule = re_render_av_rule(0,ep->ex_rules[i], 0, policy);
		if(rule == NULL) {
			fprintf(stderr, "problem rendering execute rule %d\n", i);
			return -1;
		}
		fprintf(outfile, "\t          (%d) %s\n",get_rule_lineno(ep->ex_rules[i],RULE_TE_ALLOW, policy), rule);
		free(rule);
	}	

	return 0;	
}
	

int test_print_tgt_dom(tgt_trans_domain_t *t, policy_t *policy)
{
	int rt, i;
	char *tgt;
	char *rule;
	extern FILE *outfile;
	llist_node_t *x;
	entrypoint_type_t *ep;
	rt = get_type_name(t->tgt_type, &tgt, policy);
	if(rt != 0) {
		fprintf(stderr, "\nproblem translating tgt_type (%d)\n", t->tgt_type);
		return -1;
	}
	fprintf(outfile, "\t%s (%d)\n", tgt, t->tgt_type);
	free(tgt);	
	
	fprintf(outfile, "\t     PROCESS TRANSITION RULES (%d rules):\n", t->num_pt_rules);
	for(i = 0; i < t->num_pt_rules; i++) {
		rule = re_render_av_rule(0,t->pt_rules[i], 0, policy);
		if(rule == NULL) {
			fprintf(stderr, "\nproblem rendering transition rule %d\n", i);
			return -1;
		}
		fprintf(outfile, "\t     (%d) %s\n", get_rule_lineno(t->pt_rules[i], RULE_TE_ALLOW, policy), rule);	
		free(rule);
	}
	fprintf(outfile, "\n\t     ENTRYPOINT FILE TYPES (%d types):\n", t->entry_types->num);
	for(x = t->entry_types->head; x != NULL; x = x->next) {
		ep = (entrypoint_type_t *)x->data;
		assert(t->src_type == ep->src_type);
		assert(t->tgt_type == ep->tgt_type);
		if(test_print_ep(ep, policy) != 0) {
			fprintf(stderr, "\nproblem printing entrypoint file type\n");
			return -1;
		}
	}
	
	fprintf(outfile, "\n");
	
	return 0;
}


int menu() {
	printf("\nSelect a command:\n");
	printf("0)  analyze domain transitions\n");
	printf("\n");
	printf("f)  set output file\n");
	printf("v)  show libapol version\n");
	printf("m)  display menu\n");
	printf("q)  quit\n");
	return 0;
}

int main(int argc, char *argv[])
{
	int rt, ac;
	char *av[2];
	char ans[81];
	extern FILE *outfile;
	char OutfileName[121];
	outfile = stdout;		/* Default output to  stdout */

	if(argc != 2 )
		goto usage;
		
	if ((yyin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "%s: cannot open policy file %s\n", argv[0], argv[1]);
		exit(1);
	}


	av[0] = NULL;
	interp = Tcl_CreateInterp();

	/* open policy.conf */
	ac = 2;
	av[1] = argv[1];
	if(Apol_OpenPolicy(NULL, interp, ac, av) != TCL_OK) {
		fprintf(stderr, Tcl_GetStringResult(interp));
		fprintf(stderr, "\n\n");
		exit(1);
	}

	/* test menu here */
	
	menu();
	for(;;) {
		printf("\nCommand (\'m\' for menu):  ");
		fgets(ans, sizeof(ans), stdin);	
		switch(ans[0]) {

		case '0':
		{
			domain_trans_analysis_t *dta;
			char *src;
			llist_node_t *x;
			
			printf("\tenter source domain type name:  ");
			fgets(ans, sizeof(ans), stdin);
			fix_string(ans, sizeof(ans));
			
			rt = determine_domain_trans(ans, &dta, policy);
			if(rt == -2) {
				fprintf(stderr, "\n%s is not a valid type name\n", ans);
				break;
			}
			else if(rt < 0) {
				fprintf(stderr, "\n error with analysis\n");
				break;
			}
			rt = get_type_name(dta->src_type, &src, policy);
			if(rt != 0) {
				free_domain_trans_analysis(dta);
				fprintf(stderr, "\nproblem translating src_type (%d)\n", dta->src_type);
				break;
			}
			fprintf(outfile, "\nSource type (%d): %s (%d transition domains)\n", dta->src_type, src, dta->tgt_domains->num);
			free(src);
			for(x = dta->tgt_domains->head; x != NULL; x = x->next) {
				rt = test_print_tgt_dom((tgt_trans_domain_t *)x->data, policy);
				if(rt != 0) {
					free_domain_trans_analysis(dta);
					break;
				}
			}
			
			
			free_domain_trans_analysis(dta);
			if (outfile != stdout) {
				fclose(outfile);
				outfile = stdout;
			}
			
		}	
			break;
		case 'f':
			printf("\nFilename for output (<CR> for screen output): ");
			fgets(OutfileName, sizeof(OutfileName), stdin);	
			OutfileName[strlen(OutfileName)-1] = '\0'; /* fix_string (remove LF) */
			if (strlen(OutfileName) == 0) outfile = stdout;
			else if ((outfile = fopen(OutfileName, "w")) == NULL) {
				fprintf (stderr, "Cannot open output file %s\n", OutfileName);
				outfile = stdout;
			}
			if (outfile != stdout) printf("\nOutput to file: %s\n", OutfileName);
			break;
		case 'v':
			printf("\n%s\n", VERSION_STRING);
			break;
		case 'q':
			ac = 1;
			if(Apol_ClosePolicy(NULL, interp, ac, av) != TCL_OK) {
				fprintf(stderr, Tcl_GetStringResult(interp));
				fprintf(stderr, "\n\n");
				exit(1);
			}
			exit(0);
			break;
		default:
			printf("\nInvalid choice\n");
			menu();
			break;
		}
	}
usage:
	printf("\nUsage: %s policy.conf_file \n", argv[0]);
	exit(1);

}

