 /* Copyright (C) 2002-2003 Tresys Technology, LLC
 * see file 'COPYING' for use and warranty information */

/* 
 * Author: mayerf@tresys.com 
 */

/* seuser_db.h
 *
 * The user database functions for seuser.
 *
 */
#ifndef _SEUSER_DB_H_
#define _SEUSER_DB_H_

#include "../libapol/policy.h"

/* flag values for seuser_data_t */
#define SEUSER_FLAG_NONE		(unsigned char) 0x00
#define SEUSER_FLAG_LOGIN_CONTEXT	(unsigned char) 0x01
#define SEUSER_FLAG_CRON_CONTEXT	(unsigned char) 0x02


/* default contexts; there are potentially two (cron and login) per user 
 * We use indexes from the assoicated apol library
 */
typedef struct d_context {
	int		role;
	int		type;
} d_context_t;
	
/* seuser DB additional (over apol) data, related to user record using the "data" 
 * ptr from user_list_t in libapol/policy.h
 */
#define DC_LOGIN 0
#define DC_CRON  1
#define DC_MAX   2
typedef struct seuser_data {
	unsigned char 	flags;			/* used to indicate various stuff */
	d_context_t	contexts[DC_MAX];	/* login and cron; flags indicate which */
} seuser_data_t;


/* user database structure; based in part on APOL's structures */
typedef struct user_db {
	int		num_users;
	user_list_t	users;
} user_db_t;


/* prototypes */
int copy_db_from_apol(user_db_t *db, policy_t *policy);
int seuser_get_user_by_name(const char *name, user_item_t **user, user_db_t *db);
int seuser_get_user_context(const char *user, int which, char **role, char **type, user_db_t *db, policy_t *policy);
int seuser_write_user_file(user_db_t *db, policy_t *policy, FILE *fp);
int seuser_write_default_context(int which, user_db_t *db, policy_t *policy, FILE *fp);
int seuser_update_user(user_item_t *user, bool_t newuser, user_db_t *db, policy_t *policy);
int seuser_remove_user(const char *name, user_db_t *db);
int is_proper_user_record(user_item_t *user, policy_t *policy);
int free_user_db(user_db_t *db);
int seuser_rename_user(const char *oldname, const char *newname, user_db_t *db);
bool_t seuser_does_user_exist(const char *name, user_db_t *db);

#endif /*_SEUSER_DB_H_ */



