 /* Copyright (C) 2002-2003 Tresys Technology, LLC
 * see file 'COPYING' for use and warranty information */

/* 
 * Author: mayerf@tresys.com 
 */

/* This file contains the main for seuser
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <tcl.h>
#include <tk.h>
#include <assert.h>
#include "../libapol/apol_tcl.h"
#include "../libseuser/seuser_tcl.h"
#include "../libapol/util.h"
#include "../libseuser/seuser_wrap.h"
#include <pwd.h>
#include <sys/types.h>

#define STARTUP_SCRIPT "se_user.tcl"
#define STRING_LENGTH_MAX 255

/* command line commands */
#define ADD		0
#define CHANGE		1
#define DELETE		2
#define RENAME		3
#define LOAD		4
#define SHOW		5

int Tcl_AppInit(Tcl_Interp *interp)
{
	char *script;
	char *args[2];
	if (Tcl_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	if (Tk_Init(interp) == TCL_ERROR) {
		return TCL_ERROR;
	}
	/* apolicy packagae initialization */
	if (Apol_Init(interp) == TCL_ERROR) {
     		return TCL_ERROR;
     	}
 
     	/* seuser package init */
     	if(Seuser_Init(interp) == TCL_ERROR) {
     		return TCL_ERROR;
     	}
     	    	
     	/* find and start the TCL scripts */
     	args[0] = NULL;
     	args[1] = STARTUP_SCRIPT;
     	if(Apol_GetScriptDir(NULL, interp, 2, args) != TCL_OK) {
     		fprintf(stderr, "Error finding TCL script: %s\n", interp->result);
     		Tcl_DeleteInterp(interp);
     		Tcl_Exit(1);
     	}
     	script = (char *)malloc(strlen(interp->result) + strlen(STARTUP_SCRIPT) + 3);
     	if(script == NULL) {
     		fprintf(stderr, "out of memory\n");
     		Tcl_DeleteInterp(interp);
     		Tcl_Exit(1);
     	}
     	sprintf(script, "%s/%s", interp->result, STARTUP_SCRIPT);
	if(Tcl_EvalFile(interp, script) != TCL_OK) {
		fprintf(stderr, "Error in StartScript (%s): %s\n", script, interp->result);
	  	Tcl_DeleteInterp(interp);
	  	Tcl_Exit(1);
	}

	free(script);
    	return TCL_OK;
}

void usage(bool_t brief)
{
	fprintf(stdout, "\n");
	fprintf(stdout, "Usage: seuser -X\n");
	fprintf(stdout, "       seuser delete [opts] username\n");
	fprintf(stdout, "       seuser add | change [opts] -R role1[,...] username\n");
	fprintf(stdout, "       seuser rename [opts] oldname newname\n");
	fprintf(stdout, "       seuser show roles | users [username]\n");
	fprintf(stdout, "       seuser load\n");
	fprintf(stdout, "\n");
	
	if(brief) {
	fprintf(stdout, "       Use \"seuser -h\" for extended help\n\n");
	}
	else {
	fprintf(stdout, "The -X form of this command will run a GUI.\n\n");
	fprintf(stdout, "The delete comamand will remove the specified user from the SE Linux policy.\n\n");
	fprintf(stdout, "The add/change commands will add/change a user and the user's policy \n");
	fprintf(stdout, "information.  Change/add options are:\n");
	fprintf(stdout, "     -R     authorized role(s)\n\n");
	fprintf(stdout, "The rename command will change the name of oldname to newname, leaving all other\n");
	fprintf(stdout, "information the same.\n\n");
	fprintf(stdout, "The show command will display users or roles currently defined in the policy.\n");
	fprintf(stdout, "You may optionally provide a specific username to see the definitions for that user.\n\n");
	fprintf(stdout, "The load command will reload the currently installed policy; must useful if -N is used.\n\n");
	fprintf(stdout, "\nOther options [opts]:\n\n");
	fprintf(stdout, "     -N        Do not reload the updated policy; otherwise changes are made and installed,\n");
	fprintf(stdout, "               but not loaded as the running kernel policy (but will be on next boot).\n");
	fprintf(stdout, "               Use the load command to mannually reload the installed policy when using\n");
	fprintf(stdout, "               this policy.\n\n");
	fprintf(stdout, "     -f        Force. Only used for add, change, and rename commands.  Allows you to add a user\n");
	fprintf(stdout, "               to the policy that is not an existing system user in the passwd file.\n");
	fprintf(stdout, "               This would be used, for example, to add the special user_u policy user.\n");
	fprintf(stdout, "\n");
	}
	return;
}


bool_t isSysUser(const char *user)
{
	/* TODO: Need to add checking for buffer overflows */ 
	bool_t rt_val;
	struct passwd *tmp = NULL;
	if(user == NULL)
		return FALSE;
		
	tmp = getpwnam(user);
	if(tmp == NULL)
		rt_val = FALSE;
	else 
		rt_val = TRUE;
	return rt_val;
}

int parse_roles(char *in, char ***roles, int *num, char *user)
{
	int cnt, i;
	char *ptr;

	if(num == NULL)
		return -1;		
	if(in == NULL) {
		*roles = NULL;
		num = 0;
		return 0;
	}
	
	if ( strcmp(in, "") == 0 ) {
		fprintf(stderr, "\nA null role is not a valid role.\n");
		return -1;
	}
	
	/* How many roles.*/
	for(cnt= 1, ptr = strpbrk(in, ","); ptr != NULL; ptr = strpbrk(++ptr, ","), cnt++) { ; }
			
	if(cnt > 0) {
		*roles = (char **)malloc(cnt * sizeof(char *));
		if(roles == NULL) {
			fprintf(stderr, "out of memory\n");
			return -1;
		}
	}
	else {
		*roles = NULL;
		num = 0;
		return 0;
	}		
	
	/* set array pointers to roles */
	i = 0;
	ptr = strtok(in, ",");
	while(ptr != NULL) {
		assert(i < cnt);
		(*roles)[i] = ptr;
		ptr = strtok(NULL, ",");
		i++;
	}
	*num = cnt;

	return 0;
}
#if 0 /* remove support for old style default contexts */

int parse_context(char *in, char **role, char **type)
{	
	int cnt;
	char *ptr;
	if(in == NULL || role == NULL || type == NULL)
		return -1;
		
	/* validate only two components*/
	for(cnt = 1, ptr = strpbrk(in, ":");  ptr != NULL; ptr = strpbrk(++ptr, ":"), cnt++) { ; }
	if(cnt!= 2) 
		return -1;

	*role = strtok(in, ":");
	*type = strtok(NULL, ":");
	if(*role == NULL || *type == NULL || strtok(NULL, ":") != NULL)
		return -1;		
	return 0;
}
#endif

int main(int argc, char *argv[])
{
	int i, rt, numroles, show_cmd, /* 0 = users, 1 = roles */
		cmd; 
	char *user = NULL, **roles, *olduser = NULL;
	char *drole = NULL, *dtype = NULL, *crole = NULL, *ctype = NULL;
	bool_t defcon = FALSE, croncon = FALSE;
	bool_t role  = FALSE, load = TRUE, force = FALSE;
	
	if (seu_init() != 0)
		exit(1);
	
	/* a very basic parser */
	if(argc < 2) 
		goto usage_err;
	/* GUI */
	else if(argv[1][0] == '-') {
		/* replaced -g with less desireable -X because -g already used by
		* one of the user[add|mod|del] commands */
		if(argv[1][1] == 'X' ) { 
			Tk_MainEx(1, argv, Tcl_AppInit, Tcl_CreateInterp());
  			exit(0);
		}
		else if(argv[1][1] == 'h') {
			usage(FALSE);
			exit(0);
		}
		else 
			goto usage_err;
	}
	/* command line */
	if(strcmp("delete", argv[1]) == 0) {
		cmd = DELETE;
	}
	else if(strcmp("add", argv[1]) == 0) {
		cmd = ADD;
	}
	else if(strcmp("change", argv[1]) == 0) {
		cmd = CHANGE;
	}
	else if(strcmp("show", argv[1]) == 0) {
		cmd = SHOW;
		if(argc < 3 )
			goto usage_err;
		if(argv[2][0] == '-' || (argc == 4 && argv[3][0] == '-')) 
			goto usage_err;
		if(strcmp("users", argv[2]) == 0) {
			show_cmd = 0;
		}
		else if(strcmp("roles", argv[2]) == 0) {
			show_cmd = 1;
			if(argc != 3)
				goto usage_err;
		}
		else {
			goto usage_err;
		}		
	}
	else if(strcmp("load", argv[1]) == 0) {
		cmd = LOAD;
		if(argc != 2)
			goto usage_err;
	}
	else if(strcmp("rename", argv[1]) == 0) {
		cmd = RENAME;
	}
	else 
		goto usage_err;

	/* parse options the old fashion way! */	
	if(cmd == SHOW) 
		i = 3;
	else
		i = 2;
	for(; i < argc; i++) {
		if(argv[i][0] != '-') {
			break;
		}
		else {
			if(cmd > RENAME)
				goto usage_err;
			switch(argv[i][1]) {
			case 'N':
				load = FALSE;
				break;
			/* replaced -r with less desireable -R because -r already used by
			 * one of the user[add|mod|del] commands */
			case 'R':
				if(role || cmd > CHANGE)
					goto usage_err;
				else
					role = TRUE;
				if(++i >= argc || argv[i][0] == '-')
					goto usage_err;
				rt = parse_roles(argv[i], &roles, &numroles, user);
				if(rt != 0) {
					fprintf(stderr, "\nerror parsing roles\n\n");
					exit(1);
				}
				break;
			case 'f':
				if(!(cmd == ADD || cmd == RENAME || cmd == CHANGE))
					goto usage_err;
				force = TRUE;
				break;
#if 0 /* remove support for old style default contexts */
			case 'd':
				if(defcon || cmd > 1)
					goto usage_err;
				else
					defcon = TRUE;
				if(++i >= argc || argv[i][0] == '-')
					goto usage_err;
				rt = parse_context(argv[i], &drole, &dtype);
				if(rt != 0) {
					fprintf(stderr, "error parsing default context\n\n");
					exit(1);
				}
				break;
			case 'c':
				if(croncon || cmd > 1)
					goto usage_err;
				else
					croncon = TRUE;
				if(++i >= argc || argv[i][0] == '-')
					goto usage_err;
				rt = parse_context(argv[i], &crole, &ctype);
				if(rt != 0) {
					fprintf(stderr, "error parsing cron context\n\n");
					exit(1);
				}
				break;
#endif
			default:
				goto usage_err;
			}
		}
	}

	/* old user for rename */
	if(cmd == RENAME) {
		olduser = argv[i];
		i++;
	}
	/* Get the user which should be the final argument except for load cmd. */	
	if(cmd != LOAD) {
		if(cmd == SHOW ) {
			if (argc == 4 ) {
				if(i != 3) {
					goto usage_err;
				}
				user = argv[i];
			}
			else {
				user = NULL;
			}
		}
		else {
			assert(cmd == ADD || cmd == CHANGE || cmd == DELETE || cmd == RENAME);
			if(i+1 != argc)
				goto usage_err;
			user = argv[i];
		}
	}				
	
	if((cmd == ADD || cmd == CHANGE) && !role) {
		fprintf(stderr, "\nRoles (-R) for user not specified\n");
		goto usage_err;	
#if 0 /* remove support for old style default contexts */
		/* If this is not an old login version, then context information is not needed. */
		if(use_old_login_contexts() == 0) {
			defcon = 0;
			croncon = 0;
		}
#endif
	}
	if(cmd == DELETE) {
		if(strcmp(user, "system_u") == 0) {
			fprintf(stderr, "\nFailed: can't remove the special system_u account\n\n");
			exit(1);
		}
		cmd = DELETE;
	}
	if(!force) {
		if(cmd == ADD || cmd == RENAME) {
			if(!isSysUser(user)) {
				fprintf(stderr, "\nFailed: trying to %s a non-system user (%s). Use -f option.\n\n",(cmd==ADD?"add":"rename to"), user);
				exit(1);
			}
		}
	}

#if 0 /* remove support for old style default contexts */				
	if((use_old_login_contexts()) && (((cmd == ADD || cmd == CHANGE)) && !(croncon || defcon))) {
			fprintf(stderr, "Must specify one or both of the default context for this system version.\n");
			goto usage_err;
	}
#endif
	
	if(cmd != SHOW) {
		if(!seu_checkCommitAccess()) {
			exit(1);
		}
	}
	if(cmd != LOAD) {
		seu_initUserdb();
	}
	

	if(force && cmd == CHANGE) {
		/* if the user doesn't exist and change -f request, change the command to a add */
		if(!seu_does_user_exist(user)) {
			cmd = ADD;
		}
	}
	
	switch(cmd) {
	case ADD: 
		rt = seu_editUser("add", user, roles, numroles, defcon, drole, dtype, croncon, crole, ctype);
		break;
	case CHANGE: 
		rt = seu_editUser("change", user, roles, numroles, defcon, drole, dtype, croncon, crole, ctype);
		break;
	case DELETE: 
		rt = seu_removeUser(user);
		break;
	case RENAME:
		rt = seu_renameUser(olduser, user);
		switch(rt) {
		case 1:	fprintf(stdout, "User (%s) is not an existing user\n", olduser);
			break;
		case 2: fprintf(stdout, "The new user name (%s) already exists\n", user);
			break;
		default:
			break;
		}
		break;
	case LOAD:
		rt = 0;
		/*load = TRUE; (load == TRUE) by default unless -N option included */
		break;
	case SHOW: {
		char *outstr;
		load = FALSE;
		if(show_cmd == 0)  {
			rt = seu_show_users(user, &outstr);
			if(rt == -2) {
				fprintf(stderr, "User (%s) is not an existing user\n", user);
			}
			if(rt == 0) {
				if(outstr == NULL)
					fprintf(stdout, "\nNo users currently defined.\n");
				else {
					fprintf(stdout, "\n%s\n", outstr);
					free(outstr);
				}
			}
		}
		else {
			rt = seu_show_roles(&outstr);
			if(rt == 0) {
				if(outstr == NULL)
					fprintf(stdout, "\nNo roles currently defined.\n");
				else {
					fprintf(stdout, "\n%s\n", outstr);
					free(outstr);
				}
			}
		}
		break;
	}
	default:
		fprintf(stderr, "unexepected err!\n");
		exit(1);
	}
	
	
	if(cmd <= RENAME && rt == 0) {
		fprintf(stdout, "committing changes....\n");
		rt = seu_commitChanges();
		if(rt != 0)
			exit(1);
		fprintf(stdout, "re-making policy...\n");
		rt = seu_remakePolicyConf();
	}
	if(rt != 0) {
		if(cmd == ADD || cmd == CHANGE)
			free(roles);
		exit(rt);
	}
	
	if(load) {
		fprintf(stdout, "loading new policy...\n");
		rt = seu_reinstallPolicy();
		if(rt != 0)
			exit(1);
	}
	fprintf(stdout, "\n");
	if(cmd < 2)
		free(roles);
	seu_exit();
	exit(0);	

usage_err:	
	usage(TRUE);
	exit(1);
}

