/*
 * Copyright (c) 2002 Russell Coker <russell@coker.com.au>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "defs.h"

int sys_security_get_sids(struct tcb *tcp)
{
	if(!entering(tcp))
	{
		unsigned long len;
		umoven(tcp, tcp->u_arg[1], sizeof(len), (char *)&len);
		if(tcp->u_rval == 0)
			dumpstr(tcp, tcp->u_arg[0], len * 4);
		else
			tprintf("\"\"");
		tprintf(", [%lu]", len);
	}
	return 0;
}

int sys_security_sid_to_context(struct tcb *tcp)
{
	if(entering(tcp))
	{
		tprintf("%d, ", (int)tcp->u_arg[0]);
	}
	else
	{
		unsigned long len;
		umoven(tcp, tcp->u_arg[2], sizeof(len), (char *)&len);
		if(tcp->u_rval == 0)
			printstr(tcp, tcp->u_arg[1], len - 1);
		else
			tprintf("\"\"");
		tprintf(", [%lu]", len);
	}
	return 0;
}

int sys_chsid(struct tcb *tcp)
{
	if(entering(tcp))
	{
		printpath(tcp, tcp->u_arg[0]);
		tprintf(", %lu", tcp->u_arg[1]);
	}
	return 0;
}

int sys_fchsid(struct tcb *tcp)
{
	if(entering(tcp))
		tprintf("%lu, %lu", tcp->u_arg[0], tcp->u_arg[1]);
	return 0;
}

int sys_chsidfs(struct tcb *tcp)
{
	if(entering(tcp))
	{
		printpath(tcp, tcp->u_arg[0]);
		tprintf(", %lu, %lu", tcp->u_arg[1], tcp->u_arg[2]);
	}
	return 0;
}
int sys_fchsidfs(struct tcb *tcp)
{
	if(entering(tcp))
		tprintf("%lu, %lu, %lu", tcp->u_arg[0], tcp->u_arg[1], tcp->u_arg[2]);
	return 0;
}

int sys_security_load_policy(struct tcb *tcp)
{
        if(entering(tcp))
	{
		printpath(tcp, tcp->u_arg[0]);
		tprintf(", %lu", tcp->u_arg[1]);
	}
	return 0;
}

int sys_security_context_to_sid(struct tcb *tcp)
{
        if(entering(tcp))
	{
		printstr(tcp, tcp->u_arg[0], tcp->u_arg[1] - 1);
		tprintf(", %lu, ", tcp->u_arg[1]);
	}
	else
	{
		unsigned long sid;
		if(tcp->u_rval == 0)
		{
			umoven(tcp, tcp->u_arg[2], sizeof(sid), (char *)&sid);
			tprintf("[%lu]", sid);
		}
		else
			tprintf("0x%lx", tcp->u_arg[2]);
	}
	return 0;
}

int sys_recvmsg(struct tcb *tcp);

int sys_recvmsg_secure(struct tcb *tcp)
{
	sys_recvmsg(tcp);
        if(!entering(tcp))
	{
		unsigned long sid, sid2;
		if(tcp->u_rval == 0)
		{
			umoven(tcp, tcp->u_arg[3], sizeof(sid), (char *)&sid);
			umoven(tcp, tcp->u_arg[4], sizeof(sid), (char *)&sid2);
			tprintf(", [%lu], [%lu]", sid, sid2);
		}
		else
			tprintf(", 0x%lx, 0x%lx", tcp->u_arg[3], tcp->u_arg[4]);
	}
	return 0;
}

int sys_accept(struct tcb *tcp);

int sys_accept_secure(struct tcb *tcp)
{
	sys_accept(tcp);
        if(!entering(tcp))
	{
		unsigned long sid;
		if(tcp->u_rval >= 0)
		{
			umoven(tcp, tcp->u_arg[3], sizeof(sid), (char *)&sid);
			tprintf(", [%lu]", sid);
		}
		else
			tprintf(", 0x%lx", tcp->u_arg[3]);
	}
	return 0;
}

int sys_open(struct tcb *tcp);

int sys_open_secure(struct tcb *tcp)
{
	sys_open(tcp);
        if(entering(tcp))
		tprintf(", %lu", tcp->u_arg[3]);
	return 0;
}

